((c-mode
  (flycheck-clang-include-path "/home/alvaropg/work/git/libgfbgraph/"
                               "/usr/include/json-glib-1.0"
                               "/usr/include/goa-1.0"
                               "/usr/lib64/goa-1.0/include"
                               "/usr/include/rest-0.7"
                               "/usr/include/dbus-1.0"
                               "/usr/lib64/dbus-1.0/include"
                               "/usr/include/gio-unix-2.0/"
                               "/usr/include/glib-2.0"
                               "/usr/lib64/glib-2.0/include"
                               "/usr/include/libsoup-2.4"
                               "/usr/include/libxml2")))
